;;; The sort package -- general sort & merge procedures
;;;
;;; Copyright (c) 1998 by Olin Shivers.
;;; You may do as you please with this code, as long as you do not delete this
;;; notice or hold me responsible for any outcome related to its use.
;;; Olin Shivers 10/98.

;;; This file just defines the general sort API in terms of some
;;; algorithm-specific calls.

;; FIXME: Why not use list-merge-sort here, too?  Benchmark this.
(: list-sort (procedure list -> list))
(define (list-sort < l)                   ; Sort lists by converting to
  (assert-type 'list-sort (procedure? <)) ; a vector and sorting that.
  (assert-type 'list-sort (pair-or-null? l))
  (let ((v (list->vector l)))
    (vector-sort! < v)
    (vector->list v)))

(: list-sort! (procedure list -> list))
(define (list-sort! < lis)
  (assert-type 'list-sort! (procedure? <))
  (assert-type 'list-sort! (pair-or-null? lis))
  (list-merge-sort! < lis))

(: list-stable-sort (procedure list -> list))
(define (list-stable-sort < lis)
  (assert-type 'list-stable-sort (procedure? <))
  (assert-type 'list-stable-sort (pair-or-null? lis))
  (list-merge-sort < lis))

(: list-stable-sort! (procedure list -> list))
(define (list-stable-sort! < lis)
  (assert-type 'list-stable-sort! (procedure? <))
  (assert-type 'list-stable-sort! (pair-or-null? lis))
  (list-merge-sort! < lis))

;; Implements both vector-sort and vector-stable-sort.
(: %vector-sort
   (symbol procedure vector #!optional fixnum fixnum -> vector))
(define (%vector-sort loc < vec . rest)
  (assert-type loc (procedure? <))
  (assert-type loc (vector? vec))
  (let*-values (((start end) (vector-start+end loc vec rest))
                ((len) (- end start))
                ((ans) (vector-copy vec start end)))
    (%vector-merge-sort! < ans 0 len (vector-copy ans))
    ans))

(: vector-sort (procedure vector #!optional fixnum fixnum -> vector))
(define vector-sort (cut %vector-sort 'vector-sort <...>))

(: vector-stable-sort
   (procedure vector #!optional fixnum fixnum -> vector))
(define vector-stable-sort
  (cut %vector-sort 'vector-stable-sort <...>))

;; Implements both vector-sort! and vector-stable-sort!
(: %vector-sort!
   (symbol procedure vector #!optional fixnum fixnum -> undefined))
(define (%vector-sort! loc < vec . rest)
  (assert-type loc (procedure? <))
  (assert-type loc (vector? vec))
  (let-values (((start end) (vector-start+end loc vec rest)))
    (%vector-merge-sort! < vec start end (vector-copy vec))))

(: vector-sort! (procedure vector #!optional fixnum fixnum -> undefined))
(define vector-sort! (cut %vector-sort! 'vector-sort! <...>))

(: vector-stable-sort!
   (procedure vector #!optional fixnum fixnum -> vector))
(define vector-stable-sort!
  (cut %vector-sort! 'vector-stable-sort! <...>))
