;;; This code is
;;;     Copyright (c) 1998 by Olin Shivers.
;;; The terms are: You may do as you please with this code, as long as
;;; you do not delete this notice or hold me responsible for any outcome
;;; related to its use.
;;;
;;; Blah blah blah. Don't you think source files should contain more lines
;;; of code than copyright notice?

;;; Reworked 2022 by wcm. Added locations and checks.

(: check-vector-range (symbol vector fixnum fixnum -> undefined))
(define (check-vector-range loc vec start end)
  (unless (<= 0 start end (vector-length vec))
    (bounds-exception loc "invalid range" vec start end)))

(: vector-start+end (symbol vector list -> fixnum fixnum))
(define (vector-start+end loc vector maybe-start+end)
  (when (> (length maybe-start+end) 2)
    (arity-exception loc maybe-start+end))
  (let-optionals maybe-start+end ((start 0)
                                  (end (vector-length vector)))
    (when (pair? maybe-start+end)
      (assert-type loc (fixnum? start))
      (assert-type loc (fixnum? end))
      (check-vector-range loc vector start end))
    (values start end)))

(: vectors-start+end-2
   (symbol vector vector list -> fixnum fixnum fixnum fixnum))
(define (vectors-start+end-2 loc vector-1 vector-2 maybe-start+end)
  (when (> (length maybe-start+end) 4)
    (arity-exception loc maybe-start+end))
  (let-optionals maybe-start+end ((start-1 0)
                                  (end-1 (vector-length vector-1))
                                  (start-2 0)
                                  (end-2 (vector-length vector-2)))
    (when (pair? maybe-start+end)
      (assert-type loc (fixnum? start-1))
      (assert-type loc (fixnum? end-1))
      (assert-type loc (fixnum? start-2))
      (assert-type loc (fixnum? end-2))
      (check-vector-range loc vector-1 start-1 end-1)
      (check-vector-range loc vector-2 start-2 end-2))
    (values start-1 end-1
	    start-2 end-2)))
